'encoding UTF-8  Do not remove or change this line!
'*******************************************************************************
'*
'*  $RCSfile: datafile.inc,v $
'*
'*  $Revision: 1.7 $
'*
'*  last change: $Author: jsk $ $Date: 2004/09/02 13:37:16 $
'*
'*  The Contents of this file are made available subject to the terms of
'*  either of the following licenses
'*
'*         - GNU Lesser General Public License Version 2.1
'*         - Sun Industry Standards Source License Version 1.1
'*
'*  Sun Microsystems Inc., October, 2000
'*
'*  GNU Lesser General Public License Version 2.1
'*  =============================================
'*  Copyright 2000 by Sun Microsystems, Inc.
'*  901 San Antonio Road, Palo Alto, CA 94303, USA
'*
'*  This library is free software; you can redistribute it and/or
'*  modify it under the terms of the GNU Lesser General Public
'*  License version 2.1, as published by the Free Software Foundation.
'*
'*  This library is distributed in the hope that it will be useful,
'*  but WITHOUT ANY WARRANTY; without even the implied warranty of
'*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'*  Lesser General Public License for more details.
'*
'*  You should have received a copy of the GNU Lesser General Public
'*  License along with this library; if not, write to the Free Software
'*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
'*  MA  02111-1307  USA
'*
'*
'*  Sun Industry Standards Source License Version 1.1
'*  =================================================
'*  The contents of this file are subject to the Sun Industry Standards
'*  Source License Version 1.1 (the License); You may not use this file
'*  except in compliance with the License. You may obtain a copy of the
'*  License at http://www.openoffice.org/license.html.
'*
'*  Software provided under this License is provided on an AS IS basis,
'*  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
'*  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
'*  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
'*  See the License for the specific provisions governing your rights and
'*  obligations concerning the Software.
'*
'*  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
'*
'*  Copyright: 2000 by Sun Microsystems, Inc.
'*
'*  All Rights Reserved.
'*
'*  Contributor(s): _______________________________________
'*
'*
'/******************************************************************************
'*
'* owner : joerg.skottke@sun.com
'*
'* short description : tools for handling of files
'*
'*******************************************************************************
' **
' #0 hGetFileData          ' older version of hGetDataFileSection (deprecated)
' #0 hGetDataFileSection   ' reads a datafile filtering comments and empty lines
' #1 hGetAllFileNames      ' gets all filenames in a directory recursively
' #0 hWipeDirContent       ' delete the content of a directory
' **
'\******************************************************************************

function hGetFileData( sFile as string , sKey as string ) as string

   ' this function is simply a wrapper for hGetDatafileSection. It allows 
   ' to ommit some required options for hGetDatafileSection but significantly
   ' reduces the flexibility. 

   dim lsTempArray( 300 ) as string
   dim iArraySize as integer

   ' This function reads a file and returns the first line containing sKey
   if ( dir( sFile ) <> "" ) then
      iArraySize = hGetDataFileSection( sFile, _
                                        lsTempArray(), _
                                        "" , _
                                        "" , _
                                        "noprint" )
      hGetFileData() = hGetValueForKeyAsString( lsTempArray() , sKey )
   else
      hGetFileData() = "Debug: The file was not found."
   endif

end function

'*******************************************************************************

function hGetDataFileSection( cFile as string, _
                           lsList() as string, _
                           cSection as string , _
                           cComment as string, _
                             cPrint as string ) as integer

   ' This function reads a datafile into a list.
   ' Comments (lines beginning with #) are removed from the list
   ' A comment can be passed to the log
   ' Furthermore a section in the source-file can be specified. Only
   ' lines within the section are returned then. The delimiter for a
   ' section is [section-name] <> [ ...] (or EOF)
   ' NOTE: instead of lsList() a temporary list should be used that is big
   '       enough to hold all the data from the datafile.

   dim sFile as string

   dim iSectionBegin as integer
   dim iSectionEnd as integer
   dim iSectionItems as integer

   ' print a comment to the logfile. Non optional parameter but might be ""
   if ( cComment <> "" ) then
      printlog( "" )
      printlog( cComment & " : " & cFile )
      printlog( "" )
   endif

   ' read the list from the file
   listread( lsList() , cFile , "utf8" )

   ' remove comments ( lines containing # )
   listclearpattern( lsList() , "#" )

   ' remove all blank lines
   listclearblank( lsList() )

   ' honor the section, if given. Non-optional parameter that can be ""
   if ( cSection <> "" ) then
      iSectionBegin = hGetStartOfSection( lsList() , cSection )
      iSectionEnd   = hGetEndOfSection( lsList() , iSectionBegin )
      iSectionItems = hGetSection( lsList() , iSectionBegin , iSectionEnd )
   endif

   ' do convertpath on all items - just in case the list contains pathitems
   listconvertpath( lsList() , true )

   ' print the current list - if desired.
   if ( lcase( cPrint ) = "print" ) then
      listprint( lsList() )
   endif

   ' return the number of items
   hGetDataFileSection() = listcount( lsList() )

end function

'*******************************************************************************

function hGetAllFileNames( byval cStartDir, clFileName() as string ) as integer

   ' this function takes a startdirectory plus a - usually - huge string-array
   ' and retrieves all files and directories that can be found below
   ' the startdirectory.

   dim cFileList( 8000 ) as string ' a temporary list to hold the files
   dim iFileCount as integer       ' the number of files found in one directory
   dim rc as integer               ' return code: number of files found
   dim i as integer     ' stores the current file-number to be worked on
   dim iFileType as integer        ' directory/file 0 = file, 16 = directory

   ' get the filelist for the current directory
   call getfilelist( cStartDir , "*" , cFileList() )

   ' get the number of files/directories found in the current directory
   iFileCount = listcount( cFileList() )
   rc = iFileCount

   ' add the files to the list.
   for i = 1 to iFileCount
      call listappend( clFileName() , cFileList( i ) )
   next i

   ' check every newly read file if it is a directory. If this is the case,
   ' recurse down into the directories. If it is not a directory, we get an error
   ' that is happily suppressed ;)
   for i = 1 to iFileCount

      cStartDir = cFileList( i )
      iFileType = getattr( cStartDir )

      ' do anything else when an error occurs.
      if( iFileType = 5 ) then
         warnlog( "Invalid procedure call for : " & cFileList( i ) )
      elseif( iFileType = 53 ) then
         warnlog( "File not found error : " & cFileList( i ) )
      elseif ( iFileType = 16 OR iFileType = 17 ) then
         rc = rc + hGetAllFileNames( cStartDir , clFileName() )
      endif

   next i

   hGetAllFileNames() = rc

end function

'*******************************************************************************

function hWipeDirContent( cStartDir as string ) as integer

   ' this function deletes all files within a directory
   ' max 1000 files. If nothing is deleted the function runs silent
   ' (no printlog)

   dim cFileList( 1000 ) as string
   dim iFileCount as integer
   dim iCurrentFile as integer
   dim cCurrentFile as string

   ' get the number and names of the files in the specified directory
   iFileCount = hGetAllFileNames( cStartDir , cFileList() )

   if( iFileCount > 0 ) then

      printlog( "" )

      for iCurrentFile = 1 to iFileCount

         ' pick a file from the list
         cCurrentFile = cFileList( iCurrentFile )

         ' put a note in the log
         printlog( " * deleting " & cFileList( iCurrentFile ) )

         ' delete the file
         try
		 	kill( cCurrentFile )
		catch
			Warnlog "Unable to delete: " & cCurrentFile
		endcatch	

      next iCurrentFile

      printlog( "" )

   endif

   ' return the number of deleted files
   hWipeDirContent() = iFileCount

end function
