'encoding UTF-8  Do not remove or change this line!
'*******************************************************************************
'*
'*  $RCSfile: filetools.inc,v $
'*
'*  $Revision: 1.13 $
'*
'*  last change: $Author: jsk $ $Date: 2005/07/29 12:28:56 $
'*
'*  The Contents of this file are made available subject to the terms of
'*  either of the following licenses
'*
'*         - GNU Lesser General Public License Version 2.1
'*         - Sun Industry Standards Source License Version 1.1
'*
'*  Sun Microsystems Inc., October, 2000
'*
'*  GNU Lesser General Public License Version 2.1
'*  =============================================
'*  Copyright 2000 by Sun Microsystems, Inc.
'*  901 San Antonio Road, Palo Alto, CA 94303, USA
'*
'*  This library is free software; you can redistribute it and/or
'*  modify it under the terms of the GNU Lesser General Public
'*  License version 2.1, as published by the Free Software Foundation.
'*
'*  This library is distributed in the hope that it will be useful,
'*  but WITHOUT ANY WARRANTY; without even the implied warranty of
'*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'*  Lesser General Public License for more details.
'*
'*  You should have received a copy of the GNU Lesser General Public
'*  License along with this library; if not, write to the Free Software
'*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
'*  MA  02111-1307  USA
'*
'*
'*  Sun Industry Standards Source License Version 1.1
'*  =================================================
'*  The contents of this file are subject to the Sun Industry Standards
'*  Source License Version 1.1 (the License); You may not use this file
'*  except in compliance with the License. You may obtain a copy of the
'*  License at http://www.openoffice.org/license.html.
'*
'*  Software provided under this License is provided on an AS IS basis,
'*  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
'*  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
'*  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
'*  See the License for the specific provisions governing your rights and
'*  obligations concerning the Software.
'*
'*  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
'*
'*  Copyright: 2000 by Sun Microsystems, Inc.
'*
'*  All Rights Reserved.
'*
'*  Contributor(s): _______________________________________
'*
'*
'/******************************************************************************
'*
'*  owner : joerg.skottke@sun.com
'*
'*  short description : Functions that work with files like deleting, loading...
'*
'*******************************************************************************
'**
' #1 hDeleteFile           ' delete a file 
' #1 hLoadFile             ' a more fault-tolerant version of hDateiOeffnen
' #1 hSaveFile             ' a function to write the file in the current format
' #1 hWaitWhileLoading     ' replaces IsItLoaded 
' #1 hHandleActivesOnLoad  ' handles possible messageboxes/dialogs when loading
' #1 hHandleInitialDialogs ' handles dialogs that can be triggered by templates
' #1 hGetApplicationFromDoc' find gApplication for a yet unknown document
'**
'\******************************************************************************

function hDeleteFile( cFile as string ) as boolean

    '///<h1>Function to delete a single file</li>
    '///<ul>

    ' this function tries to delete a file and does some very basic error-
    ' handling. Returns 'true' on success, only error while deleting returns
    ' 'false', if the file does not exist, it is considered to be successfully
    ' deleted.
    ' i introduced this function due to a number of cases where deleting files
    ' actually failed because of weird code or situations where the user lacks
    ' accessrights to files are not handled at all.

    '///<li>Check that the file exists</li>
    if ( dir( cFile ) <> "" ) then

        '///<li>Use kill to delete, verify the outcome</li>
        try

            kill( cFile )

            ' verify that the file does not exist anymore
            if ( dir( cFile ) = "" ) then
                printlog( "hDeleteFile::File successfully deleted." )
                hDeleteFile() = true
            else
                warnlog( "hDeleteFile::File was not deleted." )
                hDeleteFile() = false
            endif

        catch

            ' in very rare cases 'kill' fails and can be handled this way
            qaerrorlog( "hDeleteFile::Deleting file failed." )
            hDeleteFile() = false

        endcatch

    else

        printlog( "hDeleteFile::Nothing to do" )
        hDeleteFile() = true

    endif

end function

'*******************************************************************************

function hLoadFile( cFile as string, bSecWarn as boolean, iTries as integer ) as boolean

    '///<h1>Function to load a given, existing file with dialoghandler</h1>
    '/// This small function tries to load a file via fileopen, thereby allowing
    '/// a given number of retries and permitting to remove the macro warning
    '///<ul>
    
    dim bFileExists as boolean
    dim bDialogsAreClosed as boolean
    dim bFileIsOpen as boolean
    dim iTry as integer
    const CFN = "hLoadFile::"
    
    printlog( CFN & cFile )
    hLoadFile() = false


    '///<li>look if the requested file exists</li>
    bFileExists = hFileExists( cFile )

    '///<li>if the workfile does not exist, return false and end the function</li>
    if ( not bFileExists ) then
        warnlog( CFN & "Requested file could not be found. Aborting" )
        exit function
    endif


    '///</li>if the file exists, try to open it</li>
    FileOpen

    Kontext "OeffnenDlg"
    Dateiname.SetText( cFile )

    '///<li>open the file if the "Open" button is available</li>
    if ( not Oeffnen.isEnabled() ) then
        warnlog( CFN & "The open-button is disabled, aborting" )

        kontext "OeffnenDlg"
        OeffnenDlg.cancel()
        exit function
    endif

    Oeffnen.click()

    '///<li>if still everything is ok, handle possible dialogs/warnings</li>

    ' hHandleActivesOnLoad will try to close a number of dialogs
    ' the function will return false in case we encounter a dialog
    ' that could not be closed correctly.
    bDialogsAreClosed = hHandleActivesOnLoad( iTries , bSecWarn )
    if ( not bDialogsAreClosed ) then
        warnlog( CFN & "Dialogs were not handled correctly, aborting" )
        exit function
    endif
         
    
    '///<li>Verify that the document is opened</li>
    ' look if the document has been loaded by now, otherwise return false
    ' this will only be executed if all dialogs have been closed by
    ' hHandleActivesOnLoad.
    bFileIsOpen = hWaitWhileLoading()
    if ( not bFileIsOpen ) then
        warnlog( CFN & "File is not open. Aborting." )
    endif
    '///</ul>

    ' if everything went well this far, we exit the function successfully
    hLoadFile() = true

end function

'*******************************************************************************

function hSaveFile( sFile as string, _
               bOverwrite as boolean, _
              bAutoSuffix as boolean ) as boolean

    '///<h1>This function saves a file, overwriting existing ones</li>
    
    ' NOTE: If we specify bOverwrite = false and the file exists, we issue a 
    '       warning but do not handle the case here.

    dim brc as boolean
    dim iDocCount as integer
    dim bFileExists as boolean
        bFileExists = false
    const CFN = "hSaveFile::"
        
    '///<ul>
    printlog( CFN & sFile )
    
    ' optimism:
    hSaveFile() = true
    
    '///<li>Count the open documents, Saving should not change the number</li>
    iDocCount = getDocumentCount()

    '///<li>Check if the file exists</li> 
    if ( app.dir( sFile ) <> "" ) then
        bFileExists = true
    endif
    
    '///<li>Check whether we may overwrite the file or not</li>
    if ( ( not bOverwrite ) and bFileExists ) then
        warnlog( CFN & "Will not overwrite existing file" )
        hSaveFile() = false
        exit function
    endif
        
    '///<li>If we are allowed to overwrite, delete the file</li>
    ' NOTE: hDeleteFile tries to delete the file, no matter if it exists or not.
    '       The function only returns false if an existing file could not be 
    '       deleted (e.g. due to accessrights)
    if ( bOverwrite ) then
        brc = hDeleteFile( sFile )
        if ( not brc ) then
            warnlog( CFN & "Delete File failed" )
        endif
    endif
    
    '///<li>Open FileSaveAs</li>
    FileSaveAs
    
    '///<li>Verify that the FileSaveAs dialog is indeed open</li>
    kontext "SpeichernDlg"
    if ( not SpeichernDlg.exists() ) then
        warnlog( CFN & "FileSaveAs failed" ) 
        hSaveFile() = false()
        exit function
    endif
    
    '///<li>Enable/disable autoextension, set filename and save</li>
    if ( bAutoSuffix ) then
        AutomatischeDateinamenserweiterung.check()
    else
        AutomatischeDateinamenserweiterung.uncheck()
    endif
    
    Dateiname.SetText( sFile )
    Speichern.Click()
    
    '///<li>Check if saving was successful</li>
    brc = IsItSaved
    if ( not brc ) then
        warnlog( CFN & "IsItLoaded claims to have failed" )
        hSaveFile() = false
        exit function
    endif
    
    '///<li>Verify we still have an unchanged number of documents</li>
    brc = hCheckDocCount( iDocCount , false )
    if ( not brc ) then
        hSaveFile() = false()
    endif
    
    '///</ul>
    
end function

'******************************************************************************

function hHandleActivesOnLoad( iTries as integer , _
                               bSecWarn as boolean ) as boolean
                               
    '///<h1>Function to handle dialogs that may pop up on load</h1>      
    '///<ol>
    '///<li>OK to create a new document</li>
    '///<li>YES to update links</li>
    '///<li>YES to execute macros</li>
    '///</ol>

    dim iTry as integer
    dim iActives as integer
    dim cMessage as string
    const CFN = "hHandleActivesOnLoad::"
    dim bSkip as boolean

    ' think positive: preset the returnvalue with "true"
    hHandleActivesOnLoad() = true

    for iTry = 1 to iTries + 2 ' Take two extra tries but warn

        ' handle two possible dialogs:
        ' 1. The warning that the user should create a new document based
        '    on this Sample/Template (the document is writeprotected)
        ' 2. The question to update links to the internet. Note: If you
        '    use proxies, they must be set correctly otherwise the result
        '    is undefined.

        ' Although this works quite well, there is a flaw that we will not
        ' get information about an "active" dialog that cannot be closed by
        ' .yes() or .ok(). If this happens, we probably run into an error 
        kontext "Active"
        if ( active.exists() ) then
           
            cMessage = active.getText()
            cMessage = hRemoveLineBreaks( cMessage )
            bSkip = false

            wait( 100 )
            

            ' ok to create a new document
            try
                active.ok()
                iActives = iActives + 1
                printlog( CFN & "MSG (" & iActives & "): " & cMessage )
                printlog( CFN & "MSG (" & iActives & "): closed with OK" )
                bSkip = true
            catch
                ' do nothing, probably wrong dialog
            endcatch

            if ( not bSkip ) then
                ' no to update links
                try
                    active.no()
                    iActives = iActives + 1
                    printlog( CFN & "MSG (" & iActives & "): " & cMessage )
                    printlog( CFN & "MSG (" & iActives & "): closed with NO" )
                catch
                    ' do nothing, probably wrong dialog
                endcatch
            endif

        endif


        ' handle the security warning if it pops open
        kontext "SecurityWarning"
        if ( SecurityWarning.exists() ) then

            ' if the security warning is NOT expected, warn
            if ( not bSecWarn ) then
                warnlog( CFN & "The security-warning was not expected." )
            endif

            try
                SecurityWarning.ok()
                printlog( CFN & "Accepted to execute macros" )
                iActives = iActives + 1
            catch
                warnlog( CFN & "Security Warning could not be closed" )
                hHandleActivesOnLoad() = false
            endcatch
               
            if ( iTry > iTries ) then
                warnlog( CFN & "There are more Messageboxes than expected!" )
            endif
               
        endif

           
        ' The filterdialog may never open. If this happens we did not
        ' recognize our fileformat correctly -> bug
        kontext "Filterdialog"
        if ( FilterDialog.exists() ) then
            warnlog( CFN & "Filterdialog is open, unknown fileformat?" )
               
            try
                ' recovering by trying not to load the file
                FilterDialog.cancel()
                sleep( 1 )
                kontext "FilterDialog"
                if ( FilterDialog.exists() ) then
                    warnlog( CFN & "FilterDialog still open->Argh." )
                else
                    printlog( CFN & "Filterdialog has been closed with .cancel()" )
                endif
                hHandleActivesOnLoad() = false
            catch
                ' if we end up here, the outcome is undefined
                warnlog( CFN & "FilterDialog.cancel() failed." )
                call exitRestartThe Office()
                hHandleActivesOnLoad() = false
            endcatch

        endif

    next iTry

    ' now see how many dialogs were allowed and how many have been closed
    ' this does not change the return value of the function
    if ( iActives > iTries ) then
        qaerrorlog( "#i47425# The test closed more dialogs than expected" )
    endif
       
end function       

'*******************************************************************************

function hHandleInitialDialogs() as integer

    '///<h1>Function to handle dialogs while opening samples/templates etc.</h1>
    '///<ul>
    
    ' NOTE: The ID of the dialog will be returned but is of limited use
    '       Function returns 0 if no dialogs have been encountered
    
    dim incident as integer
        incident = 0

    '///<li>Test for UseOfThisTemplate-dialog</ul>
    Kontext "UseOfThisTemplate"
    if ( UseOfThisTemplate.Exists() ) then
        printlog( "hHandleInitialDialogs::Closing UseOfThisTemplate-dialog" )
        incident = 1
        CancelButton.Click()
    endif
      
    '///<li>Test for StarOfficeCalendar1-dialog</ul>
    Kontext "StarOfficeCalendar1"
    if ( StarOfficeCalendar1.Exists() ) then
        printlog( "hHandleInitialDialogs::Closing StarOfficeCalendarl-dialog" )
        incident = 2
        Create.Click()
    endif
      
    '///<li>Test for NewsletterLayout-dialog</ul>
    Kontext "NewsletterLayout"
    if ( NewsletterLayout.Exists() ) then
        printlog( "hHandleInitialDialogs::Closing NewsletterLayout-dialog" )
        incident = 3
        OkButton.Click()
    endif
      
    '///<li>Test for PortfolioCurrency-dialog</ul>
    Kontext "PortfolioCurrency"
    if ( PortfolioCurrency.Exists() ) then
        printlog( "hHandleInitialDialogs::Closing PortfolioCurrency-dialog" )
        incident = 4
        OKButton.Click()
    endif
      
    hHandleInitialDialogs() = incident
    
    '///</ul>

end function         

'*******************************************************************************

function hGetApplicationFromDoc() as integer

    '///<h1>Function to determine gApplication from unknown doctype</h1>
    '///<b>USE WITH CAUTION: Function returns "WRITER" for all writer-apps</b>
    
    ' Implementation details
    ' The function steps through all known DocumentXXXX kontexts. On first
    ' hit (DocumentXXX.exists() = true) the number is returned and the 
    ' gApplication is set. 
    ' The function relies on the existence of the initial document ("The first
    ' doc!" which *must* be a writer (gApplication = "WRITER").
    ' DocumentWriter is the default and fallback for HTML and Master-Document
    ' and thus returns gApplication = "WRITER" for all three writer related 
    ' documenttypes.
    ' Please note that the value returned matches those from hGetNumericDoctype
    
    const CFN = "hGetApplicationFromDoc::"
    dim brc as boolean
    brc = hCheckDocCount( 2 , false )
    if ( not brc ) then
        warnlog( CFN & "One of the two documents is missing" )
        hGetApplicationFromDoc() = 0
        exit function
    endif

    kontext "DocumentCalc"
    if ( DocumentCalc.exists() ) then
        gApplication = "CALC"
        hGetApplicationFromDoc() = 2
        printlog( CFN & gApplication )
        exit function
    endif

    kontext "DocumentImpress"
    if ( DocumentImpress.exists() ) then
        gApplication = "IMPRESS"
        hGetApplicationFromDoc() = 3
        printlog( CFN & gApplication )
        exit function
    endif

    kontext "DocumentDraw"
    if ( DocumentDraw.exists() ) then
        gApplication = "DRAW"
        hGetApplicationFromDoc() = 4
        printlog( CFN & gApplication )
        exit function
    endif

    kontext "DocumentMath"
    if ( DocumentMath.exists() ) then
        gApplication = "MATH"
        hGetApplicationFromDoc() = 5
        printlog( CFN & gApplication )
        exit function
    endif

    kontext "DocumentWriter"
    if ( DocumentWriter.exists() ) then
        gApplication = "WRITER"
        hGetApplicationFromDoc() = 1
        printlog( CFN & gApplication )
        'qaerrorlog( CFN & "Document may be WRITER, MASTERDOC or HTML!" )
        exit function
    endif

end function

'**************************************************************************
    
function hWaitWhileLoading() as boolean

    '///<h1>Wait for a document to load. Replaces IsItLoaded</h1>
    ' IsItLoaded wastes at least 5 seconds on each call which is fatal
    ' for the extras test which run at least 20 minutes longer than 
    ' necessary. The function works the same way as IsItLoaded but
    ' eliminates a lot of sleep-cycles.

    dim iNextProbe as integer
    dim brc as boolean
    dim iTime as integer

    const CFN = "hWaitWhileLoading::"
    const I_MAX_WAIT = 100
    const I_PROBE_INTERVAL = 100


    for iNextProbe = 1 to I_MAX_WAIT
    
        brc = false
        wait( I_PROBE_INTERVAL )

        brc = IsDocLoading

        if ( brc ) then
            hWaitWhileLoading() = true
            iTime = iNextProbe * I_PROBE_INTERVAL
            printlog( CFN & "File loaded in less than " & iTime & " ms"
            exit for
        endif

    next iNextProbe

    if ( not brc ) then
        warnlog( CFN & "File could not be loaded within 10 seconds." )
        hWaitWhileLoading() = true
    endif

end function
